/*============================================================================================
    
    ---------------------------------
    -*- [ZombiePlague] Bank       -*-
    ---------------------------------
    
    
============================================================================================*/


#include <amxmodx>
#include <cstrike>
#include <auth_core>
#include <zp50_ammopacks>
#include <sqlx>

/*==========================================================================================*/
#define PLUG_OBJNAME            "ZP AmmoBank"
#define PLUG_VERSION            "1.0"
#define PLUG_CREATOR            "Boec[SpecOPs]"

#define DB_HOST "localhost"
#define DB_USER "root"
#define DB_PASS "root"
#define DB_NAME "zombielite"

/*==========================================================================================*/

new const BANK_CREATE[] =                       
    "   CREATE TABLE IF NOT EXISTS zp_bank          \
        (                                           \
            uID INT(10) UNSIGNED PRIMARY KEY,       \
            ammo INT(10) UNSIGNED DEFAULT 0,        \
            money INT(10) UNSIGNED DEFAULT 0        \
        ) ENGINE = InnoDB;";

new Handle:db_tuple;
new fwd_bank_save, fwd_bank_load;

/*==========================================================================================*/

enum _:BankRecord {
    br_uID,    
    br_ammo,
    br_money
}

/*=========================================================================================*/

public plugin_init() 
{
    register_plugin(PLUG_OBJNAME, PLUG_VERSION, PLUG_CREATOR);
    
    fwd_bank_save = CreateMultiForward("zp_bank_save", ET_IGNORE, FP_CELL, FP_CELL, FP_CELL, FP_CELL);
    fwd_bank_load = CreateMultiForward("zp_bank_load", ET_IGNORE, FP_CELL, FP_CELL, FP_CELL, FP_CELL);
    
    mysql_db_init();
}

mysql_db_init() 
{
    db_tuple = SQL_MakeDbTuple(DB_HOST, DB_USER, DB_PASS, DB_NAME);
    SQL_ThreadQuery(db_tuple, "mysql_query_handler", BANK_CREATE);
}


/*==========================================================================================*/

public mysql_query_handler(failstate, Handle:query, error[], errnum, data[], size, Float:queuetime) 
{
    if(failstate != TQUERY_SUCCESS)    server_print("[MySQL] SQL error [%d]: %s", errnum, error);
}

public auth_status_change(pID, new_status, old_status, uID) 
{
    if(new_status == AUTH_SUCCESS) {
        bank_withdraw(pID, uID);
    } else if(old_status == AUTH_SUCCESS) {
        bank_deposit(pID, uID);
    }
}

/*==========================================================================================*/

bank_deposit(pID, uID) 
{
    new query[1024];
    new ammo = zp_ammopacks_get(pID);
    new money = cs_get_user_money(pID);
    new res;
    
    // updating information
    ExecuteForward(fwd_bank_save, res, pID, uID, ammo, money);
    ammo = zp_ammopacks_get(pID);
    money = cs_get_user_money(pID);

    format(query, charsmax(query), "REPLACE INTO zp_bank(uID, ammo, money) VALUES(%d, %d, %d);", uID, ammo, money);
    zp_ammopacks_set(pID, 0);
    
    SQL_ThreadQuery(db_tuple, "mysql_query_handler", query);
}

bank_withdraw(pID, uID) 
{
    new query[1024];
    new data[1]; data[0]=pID;
    
    format(query, charsmax(query), "SELECT uID, ammo, money FROM zp_bank WHERE uID=%d;", uID);
    
    SQL_ThreadQuery(db_tuple, "mysql_withdraw", query, data, 1);
}

public mysql_withdraw(failstate, Handle:query_results, error[], errnum, data[], size, Float:queuetime) 
{
    if(failstate != TQUERY_SUCCESS) {
        server_print("[MySQL] SQL error [%d]: %s", errnum, error);
        return;
    }
    
    new results[BankRecord];
    if(SQL_MoreResults(query_results)) {
        results = parse_results(query_results);    
    }
    new res;
    server_print("uID: %d; pID %d; ammo: %d", results[br_uID], data[0], results[br_ammo] );
    zp_ammopacks_set(data[0], results[br_ammo]);
    //cs_set_user_money(data[0], results[br_money]);
    
    // updating information
    ExecuteForward(fwd_bank_save, res, data[0], results[br_uID], results[br_ammo], results[br_money]);
}

parse_results(Handle:sql_results) 
{
    new results[BankRecord];
    new parsed = 0;
    
    results[br_uID] = SQL_ReadResult(sql_results, parsed);
    parsed += 1;
    results[br_ammo] = SQL_ReadResult(sql_results, parsed);
    parsed += 1;
    results[br_money] = SQL_ReadResult(sql_results, parsed);
    parsed += 1;
    
    return results;
}
